@@ -1,4 +1,4 @@
 cargo bundle --release
 #cargo bundle
 #cp res/info.plist target/debug/bundle/osx/oculante.app/Contents/Info.plist
-cp res/info.plist target/release/bundle/osx/oculante.app/Contents/Info.plist
\ No newline at end of file
+cp Info.plist target/release/bundle/osx/oculante.app/Contents/Info.plist
\ No newline at end of file
